@@ -5,7 +5,7 @@ Convert L{pydoctor.epydoc} parsed markup into renderable content.
 from collections import defaultdict
 from typing import (
     TYPE_CHECKING, Callable, ClassVar, DefaultDict, Dict, Generator, Iterable,
-    Iterator, List, Mapping, Optional, Sequence, Tuple, Union
+    Iterator, List, Mapping, Optional, Sequence, Set, Tuple, Union, cast
 )
 import ast
 import itertools
@@ -22,6 +22,7 @@ from pydoctor.epydoc.markup._pyval_repr import colorize_pyval, colorize_inline_p
 
 if TYPE_CHECKING:
     from twisted.web.template import Flattenable
+    from typing_extensions import Literal
 
 def get_parser(obj: model.Documentable) -> Callable[[str, List[ParseError], bool], ParsedDocstring]:
     """
@@ -52,7 +53,9 @@ def get_docstring(
     return None, None
 
 
-def taglink(o: model.Documentable, page_url: str, label: Optional["Flattenable"] = None) -> Tag:
+def taglink(o: model.Documentable, page_url: str, 
+            label: Optional["Flattenable"] = None, 
+            same_page_optimization:bool=True) -> Tag:
     """
     Create a link to an object that exists in the system.
     """
@@ -63,7 +66,7 @@ def taglink(o: model.Documentable, page_url: str, label: Optional["Flattenable"]
         label = o.fullName()
 
     url = o.url
-    if url.startswith(page_url + '#'):
+    if url.startswith(page_url + '#') and same_page_optimization is True:
         # When linking to an item on the same page, omit the path.
         # Besides shortening the HTML, this also avoids the page being reloaded
         # if the query string is non-empty.
@@ -74,11 +77,23 @@ def taglink(o: model.Documentable, page_url: str, label: Optional["Flattenable"]
         ret(title=o.fullName())
     return ret
 
-
+# TODO: Move this code to it's own module to avoid cyclic imports.
 class _EpydocLinker(DocstringLinker):
 
-    def __init__(self, obj: model.Documentable):
+    class LookupFailed(LookupError):
+        """
+        Encapsulate a link tag that is not actually a link because we count not resolve the name. 
+
+        Used only if L{_EpydocLinker.strict} is True.
+        """
+        def __init__(self, *args: object, link: Tag) -> None:
+            super().__init__(*args)
+            self.link: Tag = link
+
+    def __init__(self, obj: 'model.Documentable', same_page_optimization:bool, strict:bool=False):
         self.obj = obj
+        self.same_page_optimization=same_page_optimization
+        self.strict=strict
 
     @staticmethod
     def _create_intersphinx_link(label:"Flattenable", url:str) -> Tag:
@@ -89,9 +104,9 @@ class _EpydocLinker(DocstringLinker):
 
     def look_for_name(self,
             name: str,
-            candidates: Iterable[model.Documentable],
+            candidates: Iterable['model.Documentable'],
             lineno: int
-            ) -> Optional[model.Documentable]:
+            ) -> Optional['model.Documentable']:
         part0 = name.split('.')[0]
         potential_targets = []
         for src in candidates:
@@ -119,31 +134,46 @@ class _EpydocLinker(DocstringLinker):
         return self.obj.system.intersphinx.getLink(name)
 
     def link_to(self, identifier: str, label: "Flattenable") -> Tag:
+        # :Raises _EpydocLinker.LookupFailed: If the identifier cannot be resolved and self.strict is True.
+        # Can return a Tag('a') or Tag('transparent') if not found
         fullID = self.obj.expandName(identifier)
 
         target = self.obj.system.objForFullName(fullID)
         if target is not None:
-            return taglink(target, self.obj.page_object.url, label)
+            return taglink(target, self.obj.page_object.url, label, 
+                           same_page_optimization=self.same_page_optimization)
 
         url = self.look_for_intersphinx(fullID)
         if url is not None:
             return self._create_intersphinx_link(label, url=url)
 
-        return tags.transparent(label)
+        link = tags.transparent(label)
+        if self.strict:
+            raise self.LookupFailed(identifier, link=link)
+        return link
 
     def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
+        # :Raises _EpydocLinker.LookupFailed: If the identifier cannot be resolved and self.strict is True.
+        # Otherwise returns a Tag('code'). 
+        # If not foud the code tag will simply contain the label as Flattenable, like:
+        # Tag('code', children=['label as Flattenable'])
+        # If the link is found it gives something like:
+        # Tag('code', children=[Tag('a', href='...', children=['label as Flattenable'])])
         xref: "Flattenable"
         try:
             resolved = self._resolve_identifier_xref(target, lineno)
-        except LookupError:
+        except LookupError as e:
             xref = label
+            if self.strict:
+                raise self.LookupFailed(str(e), link=tags.code(xref)) from e
         else:
-            if isinstance(resolved, model.Documentable):
-                xref = taglink(resolved, self.obj.page_object.url, label)
-            else:
+            if isinstance(resolved, str):
                 xref = self._create_intersphinx_link(label, url=resolved)
-        ret: Tag = tags.code(xref)
-        return ret
+            else:
+                xref = taglink(resolved, self.obj.page_object.url, label, 
+                           same_page_optimization=self.same_page_optimization)
+                
+        return tags.code(xref)
 
     def resolve_identifier(self, identifier: str) -> Optional[str]:
         fullID = self.obj.expandName(identifier)
@@ -157,7 +187,7 @@ class _EpydocLinker(DocstringLinker):
     def _resolve_identifier_xref(self,
             identifier: str,
             lineno: int
-            ) -> Union[str, model.Documentable]:
+            ) -> Union[str, 'model.Documentable']:
         """
         Resolve a crossreference link to a Python identifier.
         This will resolve the identifier to any reasonable target,
@@ -197,7 +227,7 @@ class _EpydocLinker(DocstringLinker):
         # Check if 'identifier' refers to an object by Python name resolution
         # in our context. Walk up the object tree and see if 'identifier' refers
         # to an object by Python name resolution in each context.
-        src: Optional[model.Documentable] = self.obj
+        src: Optional['model.Documentable'] = self.obj
         while src is not None:
             target = src.resolveName(identifier)
             if target is not None:
@@ -219,6 +249,9 @@ class _EpydocLinker(DocstringLinker):
         # names an object in each one.  Again, if more than one object is
         # found, complain.
         target = self.look_for_name(
+            # FIXME: In order to move linker code to another module and avoid cyclic imports
+            # it should not directly use the pydoctor.model module. So system.objectsOfType should
+            # support passing the type as string.
             identifier, self.obj.system.objectsOfType(model.Module), lineno)
         if target is not None:
             return target
@@ -233,6 +266,197 @@ class _EpydocLinker(DocstringLinker):
         raise LookupError(identifier)
 
 
+class _CachedEpydocLinker(_EpydocLinker):
+    """
+    This linker implements smart caching functionalities on top of public methods defined in L{DocstringLinker}.
+
+    The cache is implemented at the L{Tag} (Stan) level, letting us do transformation over cached L{Tag} instances
+    and recycle already resolved URLs and adjust them to change formatting as requested by link_xref(). 
+    """
+    
+    @attr.s(auto_attribs=True)
+    class CacheEntry:
+        name: str
+        label: "Flattenable"
+        link: Tag
+        lookup_failed: bool
+        warned_linenos: Set[int] = attr.ib(factory=set)
+
+    class NewDerivedEntry(Exception):
+        def __init__(self, *args: object, entry:'_CachedEpydocLinker.CacheEntry') -> None:
+            super().__init__(*args)
+            self.entry=entry
+
+    _CacheType = Dict[str, Dict[bool, List['_CachedEpydocLinker.CacheEntry']]]
+    _defaultCache: _CacheType = defaultdict(lambda:{True:[], False:[]})
+
+    def __init__(self, obj: 'model.Documentable', same_page_optimization:bool=True) -> None:
+        super().__init__(obj, same_page_optimization, strict=True)
+        
+        self._link_to_cache: '_CachedEpydocLinker._CacheType' = self._defaultCache.copy()
+        self._link_xref_cache: '_CachedEpydocLinker._CacheType' = self._defaultCache.copy()
+    
+    def _get_cache(self, cache_kind: 'Literal["link_to", "link_xref"]' = "link_to") -> '_CachedEpydocLinker._CacheType':
+        cache_dict = getattr(self, f"_{cache_kind}_cache")
+        assert isinstance(cache_dict, dict)
+        return cast('_CachedEpydocLinker._CacheType', cache_dict)
+
+    def _new_derived_entry(self, 
+                             cached_entry: '_CachedEpydocLinker.CacheEntry', 
+                             label: Optional["Flattenable"], 
+                             cache_kind: 'Literal["link_to", "link_xref"]' = "link_to") -> '_CachedEpydocLinker.CacheEntry':
+
+        # Transform the URL to omit the filename when self.same_page_optimization is True and
+        # add it when self.same_page_optimization is False.
+        link = self._adjust_link(cached_entry.link, 
+                                        # here we clone the link because we need to change the label anyway
+                                        self.same_page_optimization) or (cached_entry.link.clone() if label else cached_entry.link)
+
+        # Change the label if needed.
+        if label:
+            link.children = [label]
+
+        return self._store_in_cache(
+                        cached_entry.name, 
+                        label if label else link.children[0], 
+                        link=link, 
+                        cache_kind=cache_kind,
+                        lookup_failed=cached_entry.lookup_failed,
+                        warned_linenos=cached_entry.warned_linenos # We do not use copy() here by design.
+                    )
+    
+    def _adjust_link(self, link: Tag, use_same_page_optimization:bool) -> Optional[Tag]:
+        # Returns a new link or None if the current link is correct.
+        if use_same_page_optimization is False:
+            if link.attributes.get('href', '').startswith("#"): # type:ignore
+                link = link.clone()
+                link.attributes['href'] = self.obj.page_object.url + link.attributes['href'] # type:ignore
+                assert not link.attributes['href'].startswith("#") # type:ignore
+                return link
+        else:
+            if link.attributes.get('href', '').startswith(self.obj.page_object.url+"#"): # type:ignore
+                link = link.clone()
+                link.attributes['href'] = link.attributes['href'][len(self.obj.page_object.url):] # type:ignore
+                assert link.attributes['href'].startswith("#") # type:ignore
+                return link
+        return None
+
+    def _lookup_cached_entry(self, target:str, label: "Flattenable", 
+                          cache_kind: 'Literal["link_to", "link_xref"]' = "link_to") -> Optional['_CachedEpydocLinker.CacheEntry']:
+        # Lookup an entry in the cache, raise NewDerivedEntry if the exact entry could not be found
+        # but we could extrapolate the correct link from the link we already had in the cache.
+        # Rerturns None if no corecponding entry has been found in the cache.
+        
+        # For xrefs, we first look into the link_to cache.
+        if cache_kind == "link_xref":
+            cached = self._lookup_cached_entry(target, label, cache_kind="link_to")
+            if cached is not None: return cached
+        
+        # Get the cached entries
+        cache = self._get_cache(cache_kind)
+        not_same_value_for_same_page_optimization = False
+        values = cache[target][self.same_page_optimization]
+        
+        # Fallback to the entries that have not the same value for same_page_optimization
+        # This is ok because we have support for these URL transformation, see _adjust_link. 
+        if not values: 
+            values = cache[target][not self.same_page_optimization]
+            not_same_value_for_same_page_optimization = True
+        
+        # Not found
+        if not values: 
+            return None
+
+        # Here we iterate, but we could transform this into a dict access for more speed.
+        # But at the same time, usually there are not a lot of different labels applied 
+        # to the same link in the same docstring, so the current behaviour is good enough.
+        for entry in values:
+            if entry.label==label: 
+                if not_same_value_for_same_page_optimization:
+                    new_entry = self._new_derived_entry(entry, None, cache_kind)
+                    raise self.NewDerivedEntry('new cache entry', entry=new_entry)
+                return entry
+        else: 
+            # Automatically infer what would be the link 
+            # with a different label
+            entry = values[0]
+            new_entry = self._new_derived_entry(entry, label, cache_kind)
+            raise self.NewDerivedEntry('new cache entry', entry=new_entry)               
+    
+    def _store_in_cache(self, target: str, 
+                        label: "Flattenable", 
+                        link: Tag,  
+                        cache_kind: 'Literal["link_to", "link_xref"]' = "link_to", 
+                        lookup_failed:bool=False, 
+                        warned_linenos: Optional[Set[int]]=None) -> '_CachedEpydocLinker.CacheEntry':
+        # Store a new resolved link in the cache.
+
+        cache = self._get_cache(cache_kind)
+        values = cache[target][self.same_page_optimization]
+        entry = self.CacheEntry(target, label, link=link, lookup_failed=lookup_failed)
+        if warned_linenos:
+            entry.warned_linenos = warned_linenos # We do not use copy() here by design.
+        values.insert(0, entry)
+        return entry
+
+    def _lookup_cached_link_to(self, target: str, label: "Flattenable") -> Optional[Tag]:
+        # Lookup a link_to() cached value.
+        try:
+            cached = self._lookup_cached_entry(target, label, cache_kind="link_to")
+        except self.NewDerivedEntry as e:
+            cached = e.entry
+        if cached:
+            return cached.link
+        return None
+
+    def link_to(self, target: str, label: "Flattenable") -> Tag:
+        link = self._lookup_cached_link_to(target, label)
+        if link is None: 
+            failed=False 
+            try:
+                link = super().link_to(target, label)
+            except self.LookupFailed as e:
+                link = e.link
+                failed=True
+            self._store_in_cache(target, label, link, 
+                                 cache_kind="link_to", 
+                                 lookup_failed=failed)
+        return link
+    
+    def _lookup_cached_link_xref(self, target: str, label: "Flattenable", lineno: int) -> Optional[Tag]:
+        # Lookup a link_xref() cached value. 
+        # Warns if the link is derived from a link that failed the URL lookup.
+        try:
+            cached = self._lookup_cached_entry(target, label, cache_kind="link_xref")
+        except self.NewDerivedEntry as e:            
+            cached = e.entry
+            # Warns onlt if the line number differs from any other values we have already in cache.
+            if cached.lookup_failed and lineno not in cached.warned_linenos:
+                self.obj.report(f'Cannot find link target for "{cached.name}"', 'resolve_identifier_xref', lineno_offset=lineno)
+                cached.warned_linenos.add(lineno) # Add lineno such that the warning does not trigger again for this line.
+        
+        if cached:
+            return cached.link
+        return None
+
+    def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
+        link: Optional["Flattenable"] = self._lookup_cached_link_xref(target, label, lineno)
+        if link is None:
+            failed=False 
+            try:
+                link = super().link_xref(target, label, lineno).children[0]
+            except self.LookupFailed as e:
+                link = e.link.children[0]
+                failed=True
+            if not isinstance(link, Tag): 
+                link = tags.transparent(link)
+            new_cached = self._store_in_cache(target, label, link, 
+                                              cache_kind="link_xref", 
+                                              lookup_failed=failed)
+            if failed:
+                new_cached.warned_linenos.add(lineno)
+        return tags.code(link)
+
 @attr.s(auto_attribs=True)
 class FieldDesc:
     """
@@ -269,7 +493,7 @@ class FieldDesc:
             else:
                 prefix = ""
 
-            name = prefix + insert_break_points(self.name)
+            name = tags.transparent(prefix, insert_break_points(self.name))
 
             stan_name = tags.span(class_="fieldArg")(name)
             if self.type:
@@ -294,7 +518,6 @@ class RaisesDesc(FieldDesc):
         yield tags.td(tags.code(self.type), class_="fieldArgContainer")
         yield tags.td(self.body or self._UNDOCUMENTED)
 
-
 def format_desc_list(label: str, descs: Sequence[FieldDesc]) -> Iterator[Tag]:
     """
     Format list of L{FieldDesc}. Used for param, returns, raises, etc.
@@ -365,7 +588,7 @@ class Field:
 
     def format(self) -> Tag:
         """Present this field's body as HTML."""
-        return self.body.to_stan(_EpydocLinker(self.source))
+        return self.body.to_stan(self.source.docstringlinker)
 
     def report(self, message: str) -> None:
         self.source.report(message, lineno_offset=self.lineno, section='docstring')
@@ -411,7 +634,7 @@ class FieldHandler:
 
     def __init__(self, obj: model.Documentable):
         self.obj = obj
-        self._linker = _EpydocLinker(self.obj)
+        self._linker = self.obj.docstringlinker
 
         self.types: Dict[str, Optional[Tag]] = {}
 
@@ -709,10 +932,19 @@ def parse_docstring(
         reportErrors(source, errs)
     return parsed_doc
 
+def ensure_parsed_docstring(obj: model.Documentable) -> Optional[model.Documentable]:
+    """
+    Currently, it's not 100% clear at what point the L{Documentable.parsed_docstring} attribute is set.
+    It can be set from the ast builder or later processing step.
+    
+    This function ensures that the C{parsed_docstring} attribute of a documentable is set to it's final value. 
 
-def format_docstring(obj: model.Documentable) -> Tag:
-    """Generate an HTML representation of a docstring"""
-
+    @returns: 
+        - If the C{obj.parsed_docstring} is set to a L{ParsedDocstring} instance: 
+          The source object of the docstring (might be different 
+          from C{obj} if the documentation is inherited).
+        - If the object is undocumented: C{None}.
+    """
     doc, source = get_docstring(obj)
 
     # Use cached or split version if possible.
@@ -731,20 +963,31 @@ def format_docstring(obj: model.Documentable) -> Tag:
     if parsed_doc is None and doc is not None:
         parsed_doc = parse_docstring(obj, doc, source)
         obj.parsed_docstring = parsed_doc
+    
+    if obj.parsed_docstring is not None:
+        return source
+    else:
+        return None
+
+def format_docstring(obj: model.Documentable) -> Tag:
+    """Generate an HTML representation of a docstring"""
+
+    source = ensure_parsed_docstring(obj)
 
     ret: Tag = tags.div
-    if parsed_doc is None:
+    if source is None:
         ret(tags.p(class_='undocumented')("Undocumented"))
     else:
+        assert obj.parsed_docstring is not None, "ensure_parsed_docstring() did not do it's job"
         try:
-            stan = parsed_doc.to_stan(_EpydocLinker(source))
+            stan = obj.parsed_docstring.to_stan(source.docstringlinker)
         except Exception as e:
             errs = [ParseError(f'{e.__class__.__name__}: {e}', 1)]
-            if doc is None:
+            if source.docstring is None:
                 stan = tags.p(class_="undocumented")('Broken description')
             else:
-                parsed_doc_plain = pydoctor.epydoc.markup.plaintext.parse_docstring(doc, errs)
-                stan = parsed_doc_plain.to_stan(_EpydocLinker(source))
+                parsed_doc_plain = pydoctor.epydoc.markup.plaintext.parse_docstring(source.docstring, errs)
+                stan = parsed_doc_plain.to_stan(source.docstringlinker)
             reportErrors(source, errs)
         if stan.tagName:
             ret(stan)
@@ -754,8 +997,9 @@ def format_docstring(obj: model.Documentable) -> Tag:
     fh = FieldHandler(obj)
     if isinstance(obj, model.Function):
         fh.set_param_types_from_annotations(obj.annotations)
-    if parsed_doc is not None:
-        for field in parsed_doc.fields:
+    if source is not None:
+        assert obj.parsed_docstring is not None, "ensure_parsed_docstring() did not do it's job"
+        for field in obj.parsed_docstring.fields:
             fh.handle(Field.from_epydoc(field, source))
     if isinstance(obj, model.Function):
         fh.resolve_types()
@@ -797,7 +1041,13 @@ def format_summary(obj: model.Documentable) -> Tag:
         parsed_doc = parse_docstring(obj, ' '.join(lines), source)
 
     try:
-        stan = parsed_doc.to_stan(_EpydocLinker(source))
+        # Disallow same_page_optimization in order to make sure we're not
+        # breaking links when including the summaries on other pages.
+        assert isinstance(source.docstringlinker, _CachedEpydocLinker)
+        source.docstringlinker.same_page_optimization = False
+        stan = parsed_doc.to_stan(source.docstringlinker)
+        source.docstringlinker.same_page_optimization = True
+    
     except Exception:
         # This problem will likely be reported by the full docstring as well,
         # so don't spam the log.
@@ -846,7 +1096,7 @@ def type2stan(obj: model.Documentable) -> Optional[Tag]:
     if parsed_type is None:
         return None
     else:
-        return parsed_type.to_stan(_EpydocLinker(obj))
+        return parsed_type.to_stan(obj.docstringlinker)
 
 def get_parsed_type(obj: model.Documentable) -> Optional[ParsedDocstring]:
     parsed_type = obj.parsed_type
@@ -940,7 +1190,7 @@ def _format_constant_value(obj: model.Attribute) -> Iterator["Flattenable"]:
         linelen=obj.system.options.pyvalreprlinelen,
         maxlines=obj.system.options.pyvalreprmaxlines)
     
-    value_repr = doc.to_stan(_EpydocLinker(obj))
+    value_repr = doc.to_stan(obj.docstringlinker)
 
     # Report eventual warnings. It warns when a regex failed to parse or the html2stan() function fails.
     for message in doc.warnings:
@@ -958,29 +1208,57 @@ def format_constant_value(obj: model.Attribute) -> "Flattenable":
     rows = list(_format_constant_value(obj))
     return tags.table(class_='valueTable')(*rows)
 
-def insert_break_points(text: str) -> str:
+def _split_indentifier_parts_on_case(indentifier:str) -> List[str]:
+
+    def split(text:str, sep:str) -> List[str]:
+        # We use \u200b as temp token to hack a split that passes the tests.
+        return text.replace(sep, '\u200b'+sep).split('\u200b')
+
+    match = re.match('(_{1,2})?(.*?)(_{1,2})?$', indentifier)
+    assert match is not None # the regex always matches
+    prefix, text, suffix = match.groups(default='')
+    text_parts = []
+    
+    if text.islower() or text.isupper():
+        # We assume snake_case or SCREAMING_SNAKE_CASE.
+        text_parts = split(text, '_')
+    else:
+        # We assume camelCase.  We're not using a regex because we also want it
+        # to work with non-ASCII characters (and the Python re module does not
+        # support checking for Unicode properties using something like \p{Lu}).
+        current_part = ''
+        previous_was_upper = False
+        for c in text:
+
+            if c.isupper() and not previous_was_upper:
+                text_parts.append(current_part)
+                current_part = ''
+            
+            current_part += c
+            previous_was_upper = c.isupper()
+        
+        if current_part:
+            text_parts.append(current_part)
+
+    if not text_parts: # the name is composed only by underscores
+        text_parts = ['']
+    
+    if prefix:
+        text_parts[0] = prefix + text_parts[0]
+    if suffix:
+        text_parts[-1] = text_parts[-1] + suffix
+
+    return text_parts
+
+# TODO: Use tags.wbr instead of zero-width spaces
+# zero-width spaces can interfer in subtle ways when copy/pasting a name.
+def insert_break_points(text: str) -> 'Flattenable':
     """
     Browsers aren't smart enough to recognize word breaking opportunities in
     snake_case or camelCase, so this function helps them out by inserting
     zero-width spaces.
     """
-    match = re.match('(__)?(.*?)(__)?$', text)
-    assert match is not None # the regex always matches
-    prefix, text, suffix = match.groups(default='')
-
-    if text.islower() or text.isupper():
-        # We assume snake_case or SCREAMING_SNAKE_CASE.
-        text_with_breaks = text.replace('_', '\u200b_')
-    else:
-        # We assume camelCase.  We're not using a regex because we also want it
-        # to work with non-ASCII characters (and the Python re module does not
-        # support checking for Unicode properties using something like \p{Lu}).
-        text_with_breaks = ''
-        previous_was_upper = False
-        for c in text:
-            if c.isupper() and not previous_was_upper:
-                text_with_breaks += '\u200b'
-            text_with_breaks += c
-            previous_was_upper = c.isupper()
-
-    return prefix + text_with_breaks + suffix
+    return '\u200b.'.join(
+                '\u200b'.join(
+                    _split_indentifier_parts_on_case(t)) 
+                        for t in text.split('.'))
