@@ -48,7 +48,7 @@ def format_decorators(obj: Union[model.Function, model.Attribute]) -> Iterator["
 
         # Colorize decorators!
         doc = colorize_inline_pyval(dec)
-        stan = doc.to_stan(epydoc2stan._EpydocLinker(obj))
+        stan = doc.to_stan(obj.docstringlinker)
         # Report eventual warnings. It warns when a regex failed to parse or the html2stan() function fails.
         for message in doc.warnings:
             obj.report(message)
@@ -435,7 +435,7 @@ class ClassPage(CommonPage):
 
     def classSignature(self) -> "Flattenable":
         r: List["Flattenable"] = []
-        _linker = epydoc2stan._EpydocLinker(self.ob)
+        _linker = self.ob.docstringlinker
         zipped = list(zip(self.ob.rawbases, self.ob.bases))
         if zipped:
             r.append('(')
