@@ -6,7 +6,7 @@ import sys
 import tempfile
 import os
 from pathlib import Path, PurePath
-from pydoctor import model, templatewriter, stanutils
+from pydoctor import model, templatewriter, stanutils, __version__
 from pydoctor.templatewriter import (FailedToCreateTemplate, StaticTemplate, pages, writer, 
                                      TemplateLookup, Template, 
                                      HtmlTemplate, UnsupportedTemplateVersion, 
@@ -14,7 +14,7 @@ from pydoctor.templatewriter import (FailedToCreateTemplate, StaticTemplate, pag
 from pydoctor.templatewriter.pages.table import ChildTable
 from pydoctor.templatewriter.summary import isClassNodePrivate, isPrivate
 from pydoctor.test.test_astbuilder import fromText
-from pydoctor.test.test_packages import processPackage
+from pydoctor.test.test_packages import processPackage, testpackages
 
 if TYPE_CHECKING:
     from twisted.web.template import Flattenable
@@ -469,3 +469,35 @@ def test_format_decorators() -> None:
                     r"""<span class="rst-variable-string">\\/:*?"&lt;&gt;|\f\v\t\r\n</span>"""
                     """<span class="rst-variable-quote">'</span>))<br />@simple_decorator"""
                     """(<wbr></wbr>max_examples=700, <wbr></wbr>deadline=None, <wbr></wbr>option=range(<wbr></wbr>10))<br />""")
+
+def test_index_contains_infos(tmp_path: Path) -> None:
+    """
+    Test if index.html contains the following informations:
+
+        - meta generator tag
+        - nav and links to modules, classes, names
+        - link to the root packages
+        - pydoctor github link in the footer
+    """
+
+    infos = (f'<meta name="generator" content="pydoctor {__version__}"',
+              '<nav class="navbar navbar-default"',
+              '<a href="moduleIndex.html"',
+              '<a href="classIndex.html"',
+              '<a href="nameIndex.html"',
+              'Or start at one of the root packages',
+              '<code><a href="allgames.html" class="internal-link">allgames</a></code>',
+              '<code><a href="basic.html" class="internal-link">basic</a></code>',
+              '<a href="https://github.com/twisted/pydoctor/">pydoctor</a>',)
+
+    system = model.System()
+    system.addPackage(testpackages / "allgames")
+    system.addPackage(testpackages / "basic")
+    system.process()
+    w = writer.TemplateWriter(tmp_path, TemplateLookup(template_dir))
+    w.writeSummaryPages(system)
+
+    with open(tmp_path / 'index.html', encoding='utf-8') as f:
+        page = f.read()
+        for i in infos:
+            assert i in page, page
